#include <WiFi.h>
#include <ESPmDNS.h>

/* Nom du réseau WiFi */
const char ssid[] = "";
/* Mot de passe du réseau WiFi */
const char pass[] = "";

#define DEBUG

#ifdef DEBUG
#define DSB(v) Serial.begin(v)
#define DP(m) Serial.print(m)
#define DPLN(m) Serial.println(m)
#else
#define DSB(v)
#define DP(m)
#define DPLN(m)
#endif

class Connection {
public:
  typedef enum { OFFLINE, WIFI_STBY, WIFI_OK, MDNS_OK } State;

private:
  static State sState;
  static uint32_t sLastDate;
  static uint32_t sPeriod;
  static char *sSsid;
  static char *sPass;
  static char *sName;

  Connection() {}
  
  static void update();

public:
  static void setup(const char *inSsid, const char *inPass, const char *inName, const uint32_t inPeriod = 1000ul);
  static void loop();
  static bool isOnline();
};

Connection::State Connection::sState = OFFLINE;
uint32_t Connection::sLastDate = 0ul;
uint32_t Connection::sPeriod = 1000ul;
char *Connection::sSsid = NULL;
char *Connection::sPass = NULL;
char *Connection::sName = NULL;

void Connection::update()
{
  switch (sState) {
    case OFFLINE:
      if (sSsid != NULL && sPass != NULL) {
        DP("Connexion à : ");
        WiFi.begin(sSsid, sPass);
        sState = WIFI_STBY;
      }
      break;
      
    case WIFI_STBY:
      if (WiFi.status() == WL_CONNECTED) {
        DP(' ');
        DPLN(sSsid);
        sState = WIFI_OK;
      } else {
        DP('.');
      }
      break;
      
    case WIFI_OK:
      if (WiFi.status() == WL_CONNECTED) {
        if (sName != NULL) {
          MDNS.begin(sName);
          DP("Demarrage de mDNS sous le nom ");
          DP(sName);
          DPLN(".local");
          sState = MDNS_OK;
        }
      } else {
        DPLN("Perte de connexion");
        sState = OFFLINE;
      }
      break;

    case MDNS_OK:
      if (WiFi.status() != WL_CONNECTED) {
        DPLN("Perte de connexion");
        sState = OFFLINE;
      }
      break;
  }
}

void Connection::setup(const char *inSsid, const char *inPass, const char *inName, const uint32_t inPeriod)
{
  sPeriod = inPeriod;
  sSsid = (char *)inSsid;
  sPass = (char *)inPass;
  sName = (char *)inName;
}

void Connection::loop()
{
  const uint32_t currentDate = millis();
  if ((currentDate - sLastDate) >= sPeriod) {
    update();
    sLastDate = currentDate;
  }
}

bool Connection::isOnline()
{ 
  return (sState == MDNS_OK);
}

void setup() {
  /* Pour le debug */
  DSB(115200);
  /* Connexion */
  Connection::setup(ssid, pass, "Eclairage");
}

void loop() {
  Connection::loop();
}
